"""entry point for the paragon CLI."""
import click
from paragon.benchmark import Paragon
from paragon.utils import Utils


@click.command()
@click.argument("code", nargs=-1)
@click.option(
    "--accuracy", "-a", required=False, help="Number of iterations", default=1
)
def cli(code: str, accuracy: int):
    """entry point for the paragon CLI.

    :param code: the code to benchmark
    :param accuracy: number of iterations
    """
    for idx, val in enumerate(code):
        click.secho(f"Benchmark #{idx + 1}", fg="white", bold=True)
        try:
            Paragon.bench(val, accuracy)
        except Exception as e:
            Utils.reset_stdout()
            click.echo(f"Error: {e}", err=True)
            continue


if __name__ == "__main__":
    cli()



"""all benchmark related logic"""

from paragon.stats import Stats
from paragon.utils import Utils
from paragon.mark import Mark
from paragon.animate import Animate


class Paragon:
    """handles code benchmarking"""
    @staticmethod
    def bench(code: str, accuracy: int):
        """benchmark function

        :param code: code to benchmark
        :accuracy: number of iterations
        """
        # don't need stdout
        Utils.redirect_stdout()

        # run once to make sure it's valid python
        try:
            exec(code)
        except Exception as e:
            raise e

        start = Mark()
        times, prev = [], None
        animate = Animate(accuracy)

        for _ in range(accuracy):
            exec(code)
            animate.next()
            times.append(prev.diff() if prev is not None else start.diff())
            prev = Mark()

        animate.done()
        Utils.reset_stdout()
        stats = Stats(times)
        stats.output()



"""all statistics related logic"""

class Stats:
    """handles statistics compute and output"""

    def __init__(self, times):
        self.times = times

    @property
    def avg(self):
        """returns the average time"""
        return sum(self.times) / len(self.times)

    @property
    def range(self):
        """returns the range (min, max) of all times"""
        return (min(self.times), max(self.times))

    def output(self):
        """prints stats to stdout"""
        print("\n✨ Finished in {:.2f} seconds. ✨".format(sum(self.times)))
        print("✨ Average time: {:.2f} seconds. ✨".format(self.avg))
        print(
            "✨ Range: {:.2f} ... {:.2f} seconds. ✨\n".format(
                self.range[0], self.range[1]
            )
        )



"""all utility related logic"""
import sys


class Utils:
    """utility methods"""

    @staticmethod
    def reset_stdout():
        """set stdout back to normal"""
        sys.stdout = sys.__stdout__

    @staticmethod
    def redirect_stdout(redirect_value=None):
        """set stdout to provided value or None"""
        sys.stdout = redirect_value



"""all animation related logic"""

from progress.bar import IncrementalBar
from progress.spinner import PixelSpinner


class Bar(IncrementalBar):
    """custom progress bar"""
    message = "Status"
    suffix = "%(percent).1f%% - ETA %(eta)ds"


class Animate:
    """handles progress bars and spinners"""

    def __init__(self, accuracy):
        self.bar = Bar(max=accuracy)
        self.spinner = PixelSpinner()

    def next(self):
        """advance the spinner and bar to the next iteration in cycle"""
        self.bar.next()
        self.spinner.next()

    def done(self):
        """complete the bar"""
        self.bar.finish()



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="paragon",
    version="0.0.1",
    author="Liam Scalzulli",
    author_email="liamscalzulli@gmail.com",
    description=("A tiny command line benchmarking utility"),
    long_description_content_type="text/markdown",
    license="BSD",
    keywords="benchmark, benchmarking",
    url="http://packages.python.org/paragon",
    project_urls={"Source Code": "https://github.com/terror/paragon"},
    packages=find_packages(),
    long_description=read("README.md"),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License"
    ],
    entry_points={"console_scripts": ["paragon = paragon.cli:cli"]},
    python_requires=">=3.7",
)



"""all time related logic"""

import time


class Mark:
    """mark subtraction, init, diff .. etc"""

    def __init__(self):
        self.time = time.perf_counter()

    def diff(self):
        """diff's the current counter with init's counter"""
        return round(time.perf_counter() - self.time, 2)

    def __sub__(self, other):
        """subtract two marks counts"""
        return round(other.time - self.time, 2)
